@@ -1,10 +1,11 @@
 
 #include <complex.h>
+#include <stdbool.h>
 
 
 enum mode_t { MAGN, MAGN_VIRIDS, CMPL, CMPL_MYGBM, PHSE, PHSE_MYGBM, REAL, FLOW };
 enum flip_t { OO, XO, OY, XY };
-enum interp_t { NLINEAR, NEAREST };
+enum interp_t { NLINEAR, NEAREST, LIINCO };
 
 extern complex float sample(int N, const float pos[N], const long dims[N], const long strs[N], enum interp_t interpolation, const complex float* in);
 
@@ -16,10 +17,13 @@ extern void draw(int X, int Y, int rgbstr, unsigned char (*rgbbuf)[Y][rgbstr / 4
 	enum mode_t mode, float scale, float winlow, float winhigh, float phrot,
 	long str, const complex float* buf);
 
-extern void update_buf(long xdim, long ydim, int N, const long dims[N],  const long strs[N], const long pos[N], enum flip_t flip, enum interp_t interpolation, double xzoom, double yzoom,
-		long rgbw, long rgbh, const complex float* data, complex float* buf);
+extern void draw_plot(int X, int Y, int rgbstr, unsigned char (*rgbbuf)[Y][rgbstr / 4][4],
+	enum mode_t mode, float scale, float winlow, float winhigh, float phrot,
+	long str, const complex float* buf);
 
-extern char* get_spec(int i);
+extern void update_buf(long xdim, long ydim, int N, const long dims[N],  const long strs[N], const long pos[N],
+		enum flip_t flip, enum interp_t interpolation, double xzoom, double yzoom, bool plot,
+		long rgbw, long rgbh, const complex float* data, complex float* buf);
 
 extern void draw_line(int X, int Y, int rgbstr, unsigned char (*rgbbuf)[Y][rgbstr / 4][4], float x0, float y0, float x1, float y1, const char (*color)[3]);
 extern void draw_grid(int X, int Y, int rgbstr, unsigned char (*rgbbuf)[Y][rgbstr / 4][4], const float (*coord)[4][2], int divs, const char (*color)[3]);
