@@ -1,15 +1,13 @@
 /* Copyright 2015-2016. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
- *
- * Author:
- *	2015-2016 Martin Uecker <martin.uecker@med.uni-goettinge.de>
  */
 
 #define _GNU_SOURCE
 #include <unistd.h>
 #include <complex.h>
 #include <stdbool.h>
+#include <math.h>
 
 #include <libgen.h>
 
@@ -47,6 +45,7 @@ struct view_s {
 	bool sync;
 
 	bool cross_hair;
+	bool status_bar;
 
 	const char* name;
 
@@ -60,6 +59,7 @@ struct view_s {
 	bool transpose;
 
 	// representation
+	bool plot;
 	enum mode_t mode;
 	double winhigh;
 	double winlow;
@@ -84,6 +84,11 @@ struct view_s {
 	long strs[DIMS];
 	const complex float* data;
 
+	// geometry
+	unsigned long geom_flags;
+	const float (*geom)[3][3];
+	const float (*geom_current)[3][3];
+
 	// widgets
 	GtkComboBox* gtk_mode;
 	GtkComboBox* gtk_flip;
@@ -140,6 +145,24 @@ static void add_text(cairo_surface_t* surface, int x, int y, int size, const cha
 }
 
 
+extern void update_geom(struct view_s* v)
+{
+	if (NULL == v->geom)
+		return;
+
+	long dims[DIMS];
+	md_select_dims(DIMS, v->geom_flags, dims, v->dims);
+
+	long strs[DIMS];
+	md_calc_strides(DIMS, strs, dims, 1);
+
+	v->geom_current = &v->geom[md_calc_offset(DIMS, strs, v->pos)];
+
+	printf("%f %f %f %f %f %f %f %f %f\n",
+		(*v->geom_current)[0][0], (*v->geom_current)[0][1], (*v->geom_current)[0][2],
+		(*v->geom_current)[1][0], (*v->geom_current)[1][1], (*v->geom_current)[1][2],
+		(*v->geom_current)[2][0], (*v->geom_current)[2][1], (*v->geom_current)[2][2]);
+}
 
 
 extern gboolean update_view(struct view_s* v)
@@ -234,6 +257,17 @@ extern void view_refresh(struct view_s* v)
 	update_view(v);
 }
 
+
+extern void view_add_geometry(struct view_s* v, unsigned long flags, const float (*geom)[3][3])
+{
+	v->geom_flags = flags;
+	v->geom = geom;
+	v->geom_current = NULL;
+
+	update_geom(v);
+}
+
+
 extern gboolean refresh_callback(GtkWidget *widget, gpointer data)
 {
 	UNUSED(widget);
@@ -316,6 +350,7 @@ extern gboolean geom_callback(GtkWidget *widget, gpointer data)
 
 	v->invalid = true;
 
+	update_geom(v);
 	update_view(v);
 
 	return FALSE;
@@ -349,34 +384,46 @@ extern gboolean window_callback(GtkWidget *widget, gpointer data)
 
 static void update_buf_view(struct view_s* v)
 {
-	update_buf(v->xdim, v->ydim, DIMS, v->dims, v->strs, v->pos, v->flip, v->interpolation, v->xzoom, v->yzoom,
-		   v->rgbw, v->rgbh, v->data, v->buf);
+	update_buf(v->xdim, v->ydim, DIMS, v->dims, v->strs, v->pos,
+		v->flip, v->interpolation, v->xzoom, v->yzoom, v->plot,
+		v->rgbw, v->rgbh, v->data, v->buf);
 }
 
 
+static const char* spec = "xyzcmnopqsfrtuvw";
 
 extern gboolean save_callback(GtkWidget *widget, gpointer data)
 {
 	UNUSED(widget);
 	struct view_s* v = data;
 	
-	// Prepare output filename
-	unsigned int bufsize = 255;
-	char name[bufsize];
-	char* cur = name;
-	const char* end = name + bufsize;
-	cur += snprintf(cur, end - cur, "%s", v->name);
-	char dir[bufsize];
-	strncpy(dir, v->name, bufsize);
+	int len = 0;
 
-	for ( int i = 0; i < DIMS; i++) {
-		if ( v->dims[i] != 1 && i != v->xdim && i != v->ydim ){
-			cur += snprintf(cur, end - cur, "_%s_%04ld", get_spec(i), v->pos[i]);
-		}
-	}
-	cur += snprintf(cur, end - cur, ".png");
+	len += snprintf(NULL, 0, "%s_", v->name);
 
-	v->dialog = gtk_file_chooser_dialog_new ("Save File",
+	for (int i = 0; i < DIMS; i++)
+		if ((v->dims[i] != 1) && (i != v->xdim) && (i != v->ydim))
+			len += snprintf(NULL, 0, "%c%04ld", spec[i], v->pos[i]);
+
+	len += snprintf(NULL, 0, ".png");
+
+	len++;
+
+	char* name = xmalloc(len);
+
+	int off = 0;
+
+	off += snprintf(name + off, len - off, "%s_", v->name);
+
+	for (int i = 0; i < DIMS; i++)
+		if ((v->dims[i] != 1) && (i != v->xdim) && (i != v->ydim))
+			off += snprintf(name + off, len - off, "%c%04ld", spec[i], v->pos[i]);
+
+	off += snprintf(name + off, len - off, ".png");
+
+
+
+	v->dialog = gtk_file_chooser_dialog_new("Save File",
                                       v->window,
 				      GTK_FILE_CHOOSER_ACTION_SAVE,
                                       "Cancel",
@@ -384,40 +431,47 @@ extern gboolean save_callback(GtkWidget *widget, gpointer data)
                                       "Save",
                                       GTK_RESPONSE_ACCEPT,
                                       NULL);
-	v->chooser = GTK_FILE_CHOOSER (v->dialog);
 
-	gtk_file_chooser_set_current_name (v->chooser, basename(name));
+	v->chooser = GTK_FILE_CHOOSER(v->dialog);
+
+	gtk_file_chooser_set_current_name(v->chooser, basename(name));
 	
+	char* dname = strdup(v->name);
+
 	// Outputfolder = Inputfolder
-	gtk_file_chooser_set_current_folder (v->chooser, dirname(dir));
-	gtk_file_chooser_set_do_overwrite_confirmation (v->chooser, TRUE);
+	gtk_file_chooser_set_current_folder(v->chooser, dirname(dname));
+	gtk_file_chooser_set_do_overwrite_confirmation(v->chooser, TRUE);
 
-	gint res = gtk_dialog_run (GTK_DIALOG (v->dialog));
+	gint res = gtk_dialog_run(GTK_DIALOG(v->dialog));
+
+	if (GTK_RESPONSE_ACCEPT == res) {
 
-	if (res == GTK_RESPONSE_ACCEPT) {
 		// export single image
-		char *filename = gtk_file_chooser_get_filename (v->chooser);
+		char *filename = gtk_file_chooser_get_filename(v->chooser);
+
 		if (CAIRO_STATUS_SUCCESS != cairo_surface_write_to_png(v->source, filename))
 			gtk_entry_set_text(v->gtk_entry, "Error: writing image file.\n");
 
 		gtk_entry_set_text(v->gtk_entry, "Saved!");
-		g_free (filename);
+
+		g_free(filename);
 	}
 
-
 	gtk_widget_destroy (v->dialog);
+
+	xfree(name);
+	xfree(dname);
+
 	return FALSE;
 }
 
+
+
 extern gboolean save_movie_callback(GtkWidget *widget, gpointer data)
 {
 	UNUSED(widget);
 	struct view_s* v = data;
 
-	unsigned int bufsize = 255;
-	char dir[bufsize];
-	strncpy(dir, v->name, bufsize);
-
 	int frame_dim = 10;
 
 	v->dialog = gtk_file_chooser_dialog_new("Export movie to folder",
@@ -430,15 +484,18 @@ extern gboolean save_movie_callback(GtkWidget *widget, gpointer data)
 						NULL);
 
 	v->chooser = GTK_FILE_CHOOSER(v->dialog);
+
+
+	char* dname = strdup(v->name);
+
 	// Outputfolder = Inputfolder
-	gtk_file_chooser_set_current_folder(v->chooser, dirname(dir));
+	gtk_file_chooser_set_current_folder(v->chooser, dirname(dname));
 
-	gint res = gtk_dialog_run (GTK_DIALOG (v->dialog));
+	gint res = gtk_dialog_run(GTK_DIALOG (v->dialog));
 
-	if (res == GTK_RESPONSE_ACCEPT) {
+	if (GTK_RESPONSE_ACCEPT == res) {
 
 		char *chosen_dir = gtk_file_chooser_get_filename(v->chooser);
-		char output_name[bufsize];
 
 		for (unsigned int f = 0; f < v->dims[frame_dim]; f++) {
 
@@ -449,10 +506,14 @@ extern gboolean save_movie_callback(GtkWidget *widget, gpointer data)
 				v->mode, 1. / v->max, v->winlow, v->winhigh, v->phrot,
 				v->rgbw, v->buf);
 
-			char suff[16];
-			snprintf(suff, 16, "/mov-%04d.png", f);
-			strncpy(output_name, chosen_dir, bufsize - 16);
-			strncat(output_name, suff, 16);
+			char output_name[256];
+			int len = snprintf(output_name, 256, "%s/mov-%04d.png", chosen_dir, f);
+
+			if (len + 1 >= sizeof(output_name)) {
+
+				gtk_entry_set_text(v->gtk_entry, "Error: writing image file.\n");
+				break;
+			}
 
 			if (CAIRO_STATUS_SUCCESS != cairo_surface_write_to_png(v->source, output_name))
 				gtk_entry_set_text(v->gtk_entry, "Error: writing image file.\n");
@@ -464,13 +525,15 @@ extern gboolean save_movie_callback(GtkWidget *widget, gpointer data)
 
 	gtk_widget_destroy (v->dialog);
 
+	xfree(dname);
+
 	return FALSE;
 }
 
 
 struct xy_s { float x; float y; };
 
-static struct xy_s pos2screen(const struct view_s* v, /*const+*/float (*pos)[DIMS])
+static struct xy_s pos2screen(const struct view_s* v, const float (*pos)[DIMS])
 {
 	float x = (*pos)[v->xdim];
 	float y = (*pos)[v->ydim];
@@ -481,9 +544,16 @@ static struct xy_s pos2screen(const struct view_s* v, /*const+*/float (*pos)[DIM
 	if ((XY == v->flip) || (OY == v->flip))
 		y = v->dims[v->ydim] - 1 - y;
 
+	// shift to the center of pixels
+	x += 0.5;
+	y += 0.5;
+
 	x *= v->xzoom;
 	y *= v->yzoom;
 
+	if (v->plot)
+		y = v->rgbh / 2;
+
 	return (struct xy_s){ x, y };
 }
 
@@ -492,8 +562,8 @@ static void screen2pos(const struct view_s* v, float (*pos)[DIMS], struct xy_s x
 	for (unsigned int i = 0; i < DIMS; i++)
 		(*pos)[i] = v->pos[i];
 
-	float x = xy.x / v->xzoom;
-	float y = xy.y / v->yzoom;
+	float x = xy.x / v->xzoom - 0.5;
+	float y = xy.y / v->yzoom - 0.5;
 
 	if ((XY == v->flip) || (XO == v->flip))
 		x = v->dims[v->xdim] - 1 - x;
@@ -501,10 +571,37 @@ static void screen2pos(const struct view_s* v, float (*pos)[DIMS], struct xy_s x
 	if ((XY == v->flip) || (OY == v->flip))
 		y = v->dims[v->ydim] - 1 - y;
 
-	(*pos)[v->xdim] = x;
-	(*pos)[v->ydim] = y;
+	(*pos)[v->xdim] = roundf(x);
+
+	if (!v->plot)
+		(*pos)[v->ydim] = roundf(y);
 }
 
+
+
+static void clear_status_bar(struct view_s* v)
+{
+	char buf = '\0';
+	gtk_entry_set_text(v->gtk_entry, &buf);
+}
+
+
+static void update_status_bar(struct view_s* v, const float (*pos)[DIMS])
+{
+	int x2 = (*pos)[v->xdim];
+	int y2 = (*pos)[v->ydim];
+
+	complex float val = sample(DIMS, *pos, v->dims, v->strs, v->interpolation, v->data);
+
+	// FIXME: make sure this matches exactly the pixel
+	char buf[100];
+	snprintf(buf, 100, "Pos: %03d %03d Magn: %.3e Val: %+.3e%+.3ei Arg: %+.2f", x2, y2,
+			cabsf(val), crealf(val), cimagf(val), cargf(val));
+
+	gtk_entry_set_text(v->gtk_entry, buf);
+}
+
+
 extern gboolean draw_callback(GtkWidget *widget, cairo_t *cr, gpointer data)
 {
 	UNUSED(widget);
@@ -520,11 +617,11 @@ extern gboolean draw_callback(GtkWidget *widget, cairo_t *cr, gpointer data)
 
 	if (v->rgb_invalid) {
 
-		draw(v->rgbw, v->rgbh, v->rgbstr, (unsigned char(*)[v->rgbw][v->rgbstr / 4][4])v->rgb,
+		(v->plot ? draw_plot : draw)(v->rgbw, v->rgbh, v->rgbstr,
+			(unsigned char(*)[v->rgbw][v->rgbstr / 4][4])v->rgb,
 			v->mode, 1. / v->max, v->winlow, v->winhigh, v->phrot,
 			v->rgbw, v->buf);
 
-
 		v->rgb_invalid = false;
 	}
 
@@ -549,6 +646,19 @@ extern gboolean draw_callback(GtkWidget *widget, cairo_t *cr, gpointer data)
 //		draw_grid(v->rgbw, v->rgbh, v->rgbstr, (unsigned char (*)[v->rgbw][v->rgbstr / 4][4])v->rgb, &coords, 4, &color_white);
 	}
 
+	if (v->status_bar) {
+
+		float posi[DIMS];
+		for (unsigned int i = 0; i < DIMS; i++)
+			posi[i] = v->pos[i];
+
+		update_status_bar(v, &posi);
+
+		for (struct view_s* v2 = v->next; v2 != v; v2 = v2->next)
+			if (v->sync && v2->sync)
+				update_status_bar(v2, &posi);
+	}
+
 	cairo_set_source_surface(cr, v->source, 0, 0);
 	cairo_paint(cr);
 	return FALSE;
@@ -575,6 +685,7 @@ struct view_s* create_view(const char* name, const long pos[DIMS], const long di
 	v->sync = true;
 
 	v->cross_hair = false;
+	v->status_bar = false;
 
 	v->name = name;
 	v->max = 1.;
@@ -587,6 +698,8 @@ struct view_s* create_view(const char* name, const long pos[DIMS], const long di
 	v->xdim = sq_dims[0];
 	v->ydim = sq_dims[1];
 
+	v->plot = false;
+
 	v->xzoom = 2.;
 	v->yzoom = 2.;
 
@@ -599,6 +712,10 @@ struct view_s* create_view(const char* name, const long pos[DIMS], const long di
 	md_calc_strides(DIMS, v->strs, dims, sizeof(complex float));
 	v->data = data;
 
+	v->geom_flags = 0ul;
+	v->geom = NULL;
+	v->geom_current = NULL;
+
 	v->winlow = 0.;
 	v->winhigh = 1.;
 	v->phrot = 0.;
@@ -637,22 +754,19 @@ extern gboolean toggle_sync(GtkToggleButton* button, gpointer data)
 }
 
 
-static void update_status_bar(struct view_s* v, /*const*/ float (*pos)[DIMS])
+extern gboolean toggle_plot(GtkToggleButton* button, gpointer data)
 {
-	int x2 = (*pos)[v->xdim];
-	int y2 = (*pos)[v->ydim];
+	UNUSED(button);
+	struct view_s* v = data;
+	v->plot = !v->plot;
 
-	(*pos)[v->xdim] = x2;
-	(*pos)[v->ydim] = y2;
+	gtk_widget_set_sensitive(GTK_WIDGET(v->gtk_mode),
+		v->plot ? FALSE : TRUE);
 
-	complex float val = sample(DIMS, *pos, v->dims, v->strs, v->interpolation, v->data);
+	v->invalid = true;
+	update_view(v);
 
-	// FIXME: make sure this matches exactly the pixel
-	char buf[100];
-	snprintf(buf, 100, "Pos: %03d %03d Magn: %.3e Val: %+.3e%+.3ei Arg: %+.2f", x2, y2,
-			cabsf(val), crealf(val), cimagf(val), cargf(val));
-
-	gtk_entry_set_text(v->gtk_entry, buf);
+	return FALSE;
 }
 
 
@@ -681,6 +795,8 @@ extern gboolean button_press_event(GtkWidget *widget, GdkEventButton *event, gpo
 	if (event->button == GDK_BUTTON_PRIMARY) {
 
 		v->cross_hair = false;
+		v->status_bar = false;
+		clear_status_bar(v);
 
 		v->rgb_invalid = true;
 		update_view(v);
@@ -689,15 +805,10 @@ extern gboolean button_press_event(GtkWidget *widget, GdkEventButton *event, gpo
 	if (event->button == GDK_BUTTON_SECONDARY) {
 
 		v->cross_hair = true;
+		v->status_bar = true;
 
 		set_position(v, v->xdim, pos[v->xdim]);
 		set_position(v, v->ydim, pos[v->ydim]);
-
-		update_status_bar(v, &pos);
-
-		for (struct view_s* v2 = v->next; v2 != v; v2 = v2->next)
-			if (v->sync && v2->sync)
-				update_status_bar(v2, &pos);
 	}
 
 	return FALSE;
@@ -795,13 +906,14 @@ extern struct view_s* window_new(const char* name, const long pos[DIMS], const l
 	v->gtk_winhigh = GTK_ADJUSTMENT(gtk_builder_get_object(builder, "winhigh"));
 
 	v->gtk_entry = GTK_ENTRY(gtk_builder_get_object(builder, "entry"));
+#if 0
 	PangoFontDescription* desc = pango_font_description_new();
 	pango_font_description_set_family(desc, "mono");
 	pango_font_description_set_weight(desc, PANGO_WEIGHT_BOLD);
 	pango_font_description_set_absolute_size(desc, 10 * PANGO_SCALE);
 	gtk_widget_override_font(GTK_WIDGET(v->gtk_entry), desc);
 	pango_font_description_free(desc);
-
+#endif
 	v->gtk_zoom = GTK_ADJUSTMENT(gtk_builder_get_object(builder, "zoom"));
 	v->gtk_aniso = GTK_ADJUSTMENT(gtk_builder_get_object(builder, "aniso"));
 
